/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
 * http://www.gnu.org/software/gnugo/ for more information.      *
 *                                                               *
 * Copyright 1999 and 2000 by the Free Software Foundation.      *
 *                                                               *
 * This program is free software; you can redistribute it and/or *
 * modify it under the terms of the GNU General Public License   *
 * as published by the Free Software Foundation - version 2.     *
 *                                                               *
 * This program is distributed in the hope that it will be       *
 * useful, but WITHOUT ANY WARRANTY; without even the implied    *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
 * PURPOSE.  See the GNU General Public License in file COPYING  *
 * for more details.                                             *
 *                                                               *
 * You should have received a copy of the GNU General Public     *
 * License along with this program; if not, write to the Free    *
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
 * Boston, MA 02111, USA                                         *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



#ifndef _SGFTREE_H
#define _SGFTREE_H 1

#include "gnugo.h"


void *xalloc(unsigned int);

/*
 * A property of an SGF node.  An SGF node is described by a linked
 * list of these.
 */

typedef struct SGFProperty_t {
  struct SGFProperty_t *next;
  short        name;        /*FIXME should be char with enumed flags*/
  char         value[1];    /*FIXME variable length, different types*/
} SGFProperty;

    
typedef struct SGFNode_t {
  SGFProperty       *props;
  struct SGFNode_t  *parent;
  struct SGFNode_t  *child;
  struct SGFNode_t  *next;
} SGFNode;

#define sgfUp(p)   (p)->parent
#define sgfNext(p) (p)->brother
#define sgfDown(p) (p)->son


/* low level functions */
SGFNode * sgfPrev(SGFNode *);
SGFNode * sgfRoot(SGFNode *);
SGFNode * sgfNewNode(void);

int sgfGetIntProperty(SGFNode * n, const char *, int *);
int sgfGetFloatProperty(SGFNode *, const char *, float *);
int sgfGetCharProperty(SGFNode *, const char *, char **);
void sgfAddProperty(   SGFNode *, const char *, const char *);
void sgfAddPropertyInt(SGFNode *, const char *, long);
void sgfAddPropertyFloat(SGFNode * n, const char *name, float v);
void sgfOverwriteProperty(SGFNode * n, const char *name, const char *text);
void sgfOverwritePropertyFloat(SGFNode * n, const char *name, float v);
void sgfOverwritePropertyInt(SGFNode * n, const char *name, int v);
SGFProperty *sgfMkProperty( const char *, const char *,
			   SGFNode *, SGFProperty *);

SGFNode * sgfAddPlay(SGFNode * n, int isblack, int movex,int movey);
SGFNode * sgfAddStone(SGFNode *node, int color, int movex,int movey);
int sgfPrintCharProperty(FILE *f, SGFNode *node, const char *name);
int sgfPrintCommentProperty(FILE *f, SGFNode *node, const char *name);
void sgfWriteResult(SGFNode *node, float score, int overwrite);
#if 0
SGFNode * sgfNodeCheck(SGFNode *);
#endif
SGFNode * sgfCircle(SGFNode *,int i,int j);
SGFNode * sgfSquare(SGFNode *,int i,int j);
SGFNode * sgfTriangle(SGFNode *node,int i,int j);
SGFNode * sgfMark(SGFNode *,int i,int j);
SGFNode * sgfAddComment(SGFNode *, const char *comment);
SGFNode * sgfBoardText(SGFNode *,int i,int j,const char *text);
SGFNode * sgfBoardChar(SGFNode * node,int i,int j, char c);
SGFNode * sgfBoardNumber(SGFNode *,int i,int j, int number);
SGFNode * sgfStartVariant(SGFNode *);
SGFNode * sgfStartVariantFirst(SGFNode * pr);

SGFNode * sgfCreateHeaderNode(int boardsize, float komi);

/* Read SGF tree from file. */
SGFNode * readsgf(char *, unsigned long);
SGFNode * readsgffile(const char *filename);

/* Write SGF tree to a file. */
void sgf_write_header(SGFNode *root, int overwrite, int seed, float komi);
int writesgf(SGFNode *root, const char *filename);
void sgfPrintAllProperties(FILE *file, SGFNode * node);
void unparse_root(FILE *file, SGFNode * node);
void unparse_node(FILE *file, SGFNode * node);
void unparse_game(FILE *file, SGFNode * node);


/* ---------------------------------------------------------------- */
/* ---                          SGFTree                         --- */
/* ---------------------------------------------------------------- */


typedef struct SGFTree_t {
  SGFNode  * root;
  SGFNode  * lastnode;
} SGFTree;


void sgftree_clear(SGFTree *tree);
int  sgftree_readfile(SGFTree *tree, const char *infilename);

SGFNode * sgftreeAddPlay(SGFTree *tree, SGFNode *node, 
			 int isblack, int movex, int movey);
SGFNode * sgftreeAddStone(SGFTree *tree, SGFNode *node,
			  int color, int movex,int movey);
void      sgftreeWriteResult(SGFTree *tree, float score, int overwrite);
SGFNode * sgftreeNodeCheck(SGFTree *tree, SGFNode *node);

SGFNode * sgftreeCircle  (SGFTree *tree, SGFNode *node, int i, int j);
SGFNode * sgftreeSquare  (SGFTree *tree, SGFNode *node, int i, int j);
SGFNode * sgftreeTriangle(SGFTree *tree, SGFNode *node, int i, int j);
SGFNode * sgftreeMark    (SGFTree *tree, SGFNode *node, int i, int j);
SGFNode * sgftreeAddComment(SGFTree *tree, SGFNode *node, const char *comment);
SGFNode * sgftreeBoardText(SGFTree *tree, SGFNode *node,
			   int i, int j,const char *text);
SGFNode * sgftreeBoardChar(SGFTree *tree, SGFNode *node, int i,int j, char c);
SGFNode * sgftreeBoardNumber(SGFTree *tree, SGFNode *node,
			     int i, int j, int number);
SGFNode * sgftreeStartVariant(SGFTree *tree, SGFNode *node);
SGFNode * sgftreeStartVariantFirst(SGFTree *tree, SGFNode *node);
SGFNode * sgftreeCreateHeaderNode(SGFTree *tree, int boardsize, float komi);
void      sgftreeSetLastNode(SGFTree *tree, SGFNode * lastnode);

#endif

